/** アカウントのコントローラ */
class AccountController {

    def authenticateService
    
    def index = { redirect(action:list,params:params) }

    def list = {
        if(!params.max) params.max = 20
        [ accountList: Account.list( params ) ]
    }

    def createFlow = {
        // 初期化処理
        start {
            action {
                flow.account = new Account()
                flow.customer = new Customer()
            }
            on("success").to "createAccount"
        }
        // アカウントの情報を入力する画面で発生するイベントの処理
        createAccount {
            // 「次へ」ボタン押下時の処理
            on("next") {
                bindData(flow.account, params)
                flow.account.validate() ? success() : error() 
            }.to "createCustomer"
        }
        // アカウントの詳細情報を入力する画面で発生するイベントの処理
        createCustomer {
        	// 「戻る」ボタン押下時の処理
            on("back").to "createAccount"
            // 「次へ」ボタン押下時の処理
            on("next") {
                bindData(flow.customer, params)
        		flow.text = generateRandomText()
                flow.customer.validate() ? success() : error() 
            }.to "confirm"
        }
        // 入力したアカウント情報を確認する画面で発生するイベントの処理
        confirm {
            // 「戻る」ボタン押下時の処理
            on("back").to "createCustomer"
            // ［登録する］ボタン押下時の処理
            on("save") {
            	if (flow.text != params.text) {
            		flow.text = generateRandomText()
            		return error()
            	}
                flow.account.password = authenticateService.passwordEncoder(flow.account.password)
                flow.account.customer = flow.customer
                flow.account.save()
                Role.findByName('ROLE_USER').addToAccounts(flow.account).save()
            }.to "complete"
        }
        complete {
        }
    }
    
    def updateFlow = {
        start {
            action {
                flow.customer = Customer.get(authenticateService.userDomain().customer.id) 
            }
            on("success").to "edit"
        }
        edit {
            on("next") {
                bindData(flow.customer, params)
                flow.customer.validate() ? success() : error() 
            }.to "confirm"
        }
        confirm {
            on("back").to "edit"
            on("save") {
            	flow.customer.save()
            	authenticateService.userDomain().customer = flow.customer
            }.to "complete"
        }
        complete {
        }
    }

    def passwordFlow = {
        start {
            action {
                flow.account = Account.get(authenticateService.userDomain().id)
                flow.account.password = null
            }
            on("success").to "edit"
        }
        edit {
            on("next") {
                bindData(flow.account, params)
                if (!flow.account.validate()) {
                	return error()
                }
                flow.account.password = authenticateService.passwordEncoder(flow.account.password)
                flow.account.save()
                authenticateService.principal().domainClass = flow.account
            }.to "complete"
        }
        complete {
        }
    }
    
    def generateRandomText() {
        org.apache.commons.lang.RandomStringUtils.randomAlphanumeric(6)
    }
    
    def checkName = {
    	if (Account.findByName(params?.name)) {
        	render "既に使用されています"
    	}
    	else {
		    render ""
    	}
    }
}